#! /usr/bin/env perl6
# -*- mode: cperl -*-

# Fancy static types!  Not necessary, but interesting. :)
subset NonNegativeInt of Int where * >= 0;

sub format-size(NonNegativeInt $size) {
  my constant @units = <B KiB MiB GiB TiB PiB EiB ZiB YiB>;
  my $order-of-magnitude = min(8, $size == 0 ?? 0 !! floor(log($size, 1024)));
  my $unit-count = floor($size/(1024**$order-of-magnitude));
  sprintf "%3s %3s", $unit-count, @units[$order-of-magnitude];
};

sub main(@argv) {
  my $directory = @argv[0] // ".";

  my @records = (qqx{du -k $directory}.lines.map: {
    /^(\d+)\s+(.*)$/ and ($0*1024 => $1);
  }).sort;

  for @records -> $record {
    my ($size, $dir) = $record.kv;
    $size = format-size $size;
    printf "%7s   %s\n", $size, $dir;
  };
}

main @*ARGS;
